#======================================================================
#	CLSP - Capacited Lot Sizing Problem
#  Planowanie wielkoci partii produkcyjnych przy ogrniczonych zasobach
#  Wyniki s drukowane do pliku *.CSV. Ich podsumowanie jest w pliku *.XLS
#======================================================================

param LiczbaDni > 0;
set Dni := 1..LiczbaDni;

set Produkty;

param Popyt 'd' {Produkty, Dni} >= 0;
param KosztMagazynowania 'h' >= 0;
param ZapasPoczatkowy 'I0' {Produkty} >= 0;
param CzasOperacji 'p' {Produkty} >= 0;

param CzasPracy 'C'>= 0;
param KosztPrzezbrojenia 's' >= 0;

#======================================================================
param  Obciazenie {t in Dni} :=
	sum {j in Produkty}
		(sum {d in Dni: d <= t} Popyt[j, d]- ZapasPoczatkowy[j])
	* CzasOperacji[j] / (t * CzasPracy);

display Obciazenie;
check {t in Dni}: Obciazenie[t] <= 1;

#======================================================================
var Przezbrojenie 'y' {Produkty, Dni} binary;
var Produkcja 'x' {Produkty, Dni} >= 0;
var Zapas 'I' {Produkty, Dni union {0}} >= 0;

#======================================================================
minimize Calkowite_koszty:
	KosztPrzezbrojenia * sum {j in Produkty, t in Dni} Przezbrojenie[j, t]
	+ KosztMagazynowania * sum {j in Produkty, t in Dni} Zapas[j, t];

#======================================================================
subject to

	Zapas_poczatkowy {j in Produkty}:
		Zapas[j, 0] = ZapasPoczatkowy[j];

	Bilans_zapasow {j in Produkty, t in Dni}:
		Zapas[j, t-1]+ Produkcja[j, t] - Popyt[j, t] = Zapas[j, t];

	Czas_pracy {t in Dni}:
		sum {j in Produkty} CzasOperacji[j] * Produkcja[j, t] <= CzasPracy;

	Uruchomienie_produkcji {t in Dni, j in Produkty}:
		Produkcja[j, t] <= CzasPracy * Przezbrojenie[j, t];

#======================================================================
solve; #

printf "Zmienna;Produkt;Dzie;Tydzie;Warto" > "CLSP.csv";
# Tydzie jest wyliczany z jako: (Miesic-1) div 7 + 1
for {j in Produkty, t in Dni} {
	printf "\nPopyt;%s;%d;%d;%d", j, t, (t-1) div 7 + 1, Popyt[j, t] >> "CLSP.csv";
	printf "\nPrzezbrojenie;%s;%d;%d;%d", j, t, (t-1) div 7 + 1, Przezbrojenie[j, t] >> "CLSP.csv";
	printf "\nProdukcja;%s;%d;%d;%4.2f", j, t, (t-1) div 7 + 1, Produkcja[j, t] >> "CLSP.csv";
	printf "\nZapas;%s;%d;%d;%4.2f", j, t, (t-1) div 7 + 1, Zapas[j, t] >> "CLSP.csv";
	}

#======================================================================
data;

param LiczbaDni := 14;

param KosztPrzezbrojenia := 500;
param KosztMagazynowania := 1;

param CzasPracy := 30;

param: Produkty:
		ZapasPoczatkowy CzasOperacji :=
	A	0	1
	B	10	1
	C	0	1
	D	5	1
;

param Popyt (tr):
		A	B	C	D	:=
	1	7	6	7	6
	2	8	1	8	1
	3	1	5	1	5
	4	10	9	10	9
	5	6	8	6	8
	6	4	7	4	7
	7	7	6	7	6
	8	8	1	8	1
	9	1	5	1	5
	10	10	9	10	9
	11	6	8	6	8
	12	4	7	4	7
	13	8	1	8	1
	14	1	5	1	5
;

#======================================================================
end;
